
comment * ------------------------------------------------------------------

    *************************************************************
    * NOTE:                                                     *
    * This macro system is not supported in MASM32 and is still *
    * in the development stage where it is subject to change.   *
    *************************************************************

                      Function declaration macros

    This macro system is designed for simplifying procedure calls
    by removing the "invoke" syntax so that it can be called
    directly by its name. It works by prepending "FN_" to
    the procedure name and then using an equate to match the
    required procedure name to the modified name.

    MyFunc equ <invoke FN_MyFunc,>

    Writing the FUNCTION is done with the FUNCTION macro and
    terminated with the ENDFN macro.

    FUNCTION MyFunc,parameters
      ; write code here
      ret
    ENDFN

    When you use the procedure, the equate is expanded up to its
    full name with the leading FN_ prepended by INVOKE. This allows,

        MyFunc Parameters

        rather than

        invoke MyFunc,parameters

    When you need to use the original procedure name there is another
    macro called FPTR which handles the leading FN_ so that if you have
    to specify the OFFSET of the procedure or use CALL, you get the
    correct address by way of the correct label name being substituted
    in the body of the code that uses it.

    Instead of using code like,

        "mov wc.lpfnWndProc, offset WndProc"

        you use

        "mov wc.lpfnWndProc, offset FPTR(WndProc)"

    --------------------------------------------------------------------- *

  ; -------------------------------------
  ; function prototype declaration macro
  ; -------------------------------------

    DECLARE MACRO cname,parameters:VARARG
      LOCAL txt,fname
      fname equ <FN_>
      fname CATSTR fname,<cname>
      txt CATSTR fname
      fname CATSTR fname,< PROTO parameters>
      fname
      cname equ <invoke txt,>
    ENDM

  ; ---------------------
  ; begin function macro
  ; ---------------------
    FUNCTION MACRO procname,parameters:VARARG
      LOCAL txt
      fn@@name@?? equ <FN_>
      fn@@name@?? CATSTR fn@@name@??,<procname>     ;; write name for ENDFN
      txt equ <FN_>
      txt CATSTR txt,<procname proc parameters>
      align 4
      txt
    ENDM

  ; -------------------
  ; end function macro
  ; -------------------
    ENDFN MACRO
      fn@@name@?? endp
    ENDM

    ENDF equ <ENDFN>

  ; -----------------------
  ; function pointer macro
  ; -----------------------
    FPTR MACRO parameter
      LOCAL txt
      txt equ <FN_>
      txt CATSTR txt,<parameter>
      EXITM <txt>
    ENDM

  ; ----------------------------
  ; function return value macro
  ; ----------------------------
    RV MACRO args:VARARG
      args
      EXITM <eax>
    ENDM